
import java.util.*;
import java.awt.geom.*;

public class TestPointList {
	static Point2D.Double[] tp = {
		new Point2D.Double(3, 5), new Point2D.Double(1, 8),
		new Point2D.Double(6, 7), new Point2D.Double(2, 6),
		new Point2D.Double(1, 3), new Point2D.Double(3, 2),
		new Point2D.Double(1, 9), new Point2D.Double(7, 3),
		new Point2D.Double(4, 8), new Point2D.Double(4, 4),
		new Point2D.Double(9, 3), new Point2D.Double(5, 8)
	};
	static final int SIMPLE = 0, TREE = 1;
	static int sl;

	public static PointList2D createPointList() {
		switch(sl) {
			case SIMPLE:
				return new SimplePointList2D();
			case TREE:
				return new Tree2D();
		}
		return null;
	}

	public static String selectionString() {
		switch(sl) {
			case SIMPLE:
				return "[Simple]/Tree";
			case TREE:
				return "Simple/[Tree]";
		}
		return null;
	}

	public static boolean checkLists(String name, ArrayList res, Point2D.Double[] cmp) {
		boolean eq = true;
		if (res.size() != cmp.length) {
			eq = false;
		}
		else {
			int size = cmp.length;
			for (int i = 0; i < size; i ++) {
				if (!res.contains(cmp[i])) {
					eq = false;
				}
			}
		}
		if (!eq) {
			Out.println("Test " + name + " failed!");
			Out.print("Result is ");
			printList(res);
			Out.println();
			Out.print("But should be ");
			printList(cmp);
			Out.println();
		}
		return eq;
	}

	public static void printList(Point2D.Double[] points) {
		int size = points.length;
		for (int i = 0; i < size; i ++) {
			Point2D.Double p = points[i];
			Out.print("|" + p.x + "," + p.y);
		}
		Out.print("|");
	}

	public static void printList(ArrayList points) {
		Iterator iter = points.iterator();
		while (iter.hasNext()) {
			Point2D.Double p = (Point2D.Double)iter.next();
			Out.print("|" + p.x + "," + p.y);
		}
		Out.print("|");
	}

	public static void main(String[] args) {
		sl = SIMPLE;
		char ch;
		Out.print("Quit|" + selectionString() + "|Functionality/Performance (q/s/t/f/p): ");
		ch = In.read();
		while (ch != 'q') {
			In.readLine();
			switch (ch) {
				case 's':
					sl = SIMPLE;
					break;
				case 't':
					sl = TREE;
					break;
				case 'f':
					testFunctionality();
					break;
				case 'p':
					testPerformance();
					break;
				default:
					Out.println("Illegal selection!");
			}
			Out.print("Quit|" + selectionString() + "|Functionality/Performance (q/s/t/f/p): ");
			ch = In.read();
		}
		Out.println("Bye!");
	}

	static void testFunctionality() {
		boolean failed = false;
		PointList2D pl = createPointList();
		pl.printAll();
		Out.println();
		pl.insert(tp[0]);
		pl.insert(tp[1]);
		pl.insert(tp[2]);
		pl.insert(tp[3]);
		pl.insert(tp[4]);
		pl.insert(tp[5]);
		pl.insert(tp[6]);
		pl.insert(tp[7]);
		pl.insert(tp[8]);
		pl.insert(tp[9]);
		pl.insert(tp[10]);
		pl.insert(tp[11]);
		pl.printAll();
		Point2D.Double p = new Point2D.Double(1, 7);
		if (pl.contains(p)) {
			failed = true;
			Out.println("Test 1 - " + p + " found, but not inserted!");
		}
		p = tp[6];
		if (!pl.contains(p)) {
			failed = true;
			Out.println("Test 2 - " + p + " not found, but inserted!");
		}
		p = new Point2D.Double(1, 9);
		if (!pl.contains(p)) {
			failed = true;
			Out.println("Test 3a - " + p + " not found, but inserted!");
		}
		p = tp[0];
		if (!pl.contains(p)) {
			failed = true;
			Out.println("Test 3b - " + p + " not found, but inserted!");
		}
		p = tp[1];
		if (!pl.contains(p)) {
			failed = true;
			Out.println("Test 3c - " + p + " not found, but inserted!");
		}
		p = tp[2];
		if (!pl.contains(p)) {
			failed = true;
			Out.println("Test 3d - " + p + " not found, but inserted!");
		}
		p = tp[10];
		if (!pl.contains(p)) {
			failed = true;
			Out.println("Test 3e - " + p + " not found, but inserted!");
		}
		p = tp[11];
		if (!pl.contains(p)) {
			failed = true;
			Out.println("Test 3f - " + p + " not found, but inserted!");
		}
		// Fuer weitere Tests


		Point2D.Double[] cmp4a = {tp[0], tp[5]};
		if (!checkLists("4a - RangeX(2,4)", pl.findInRangeX(2, 4), cmp4a)) {
			failed = true;
		}
		Point2D.Double[] cmp4b = {tp[3]};
		if (!checkLists("4b - RangeX(1.5,2.5)", pl.findInRangeX(1.5, 2.5), cmp4b)) {
			failed = true;
		}
		Point2D.Double[] cmp4c = {};
		if (!checkLists("4c - RangeX(7.5,8.5)", pl.findInRangeX(7.5, 8.5), cmp4c)) {
			failed = true;
		}
		Point2D.Double[] cmp4d = {tp[0], tp[3], tp[5], tp[8], tp[9]};
		if (!checkLists("4d - RangeX(1.9,4.1)", pl.findInRangeX(1.9, 4.1), cmp4d)) {
			failed = true;
		}
		// Fuer weitere Tests


		Point2D.Double[] cmp5a = {tp[4], tp[7], tp[10]};
		if (!checkLists("5a - RangeY(2,4)", pl.findInRangeY(2, 4), cmp5a)) {
			failed = true;
		}
		Point2D.Double[] cmp5b = {tp[5]};
		if (!checkLists("5b - RangeY(1.5,2.5)", pl.findInRangeY(1.5, 2.5), cmp5b)) {
			failed = true;
		}
		Point2D.Double[] cmp5c = {};
		if (!checkLists("5c - RangeY(0,1.5)", pl.findInRangeY(0, 1.5), cmp5c)) {
			failed = true;
		}
		Point2D.Double[] cmp5d = {tp[4], tp[5], tp[7], tp[9], tp[10]};
		if (!checkLists("5d - RangeY(1.9,4.1)", pl.findInRangeY(1.9, 4.1), cmp5d)) {
			failed = true;
		}
		// Fuer weitere Tests

		Point2D.Double[] cmp6a = {tp[5], tp[9]};
		if (!checkLists("6a - InRect(2,2,2.5,2.5)", pl.findInRectangle(new Rectangle2D.Double(2, 2, 2.5, 2.5)), cmp6a)) {
			failed = true;
		}
		Point2D.Double[] cmp6b = tp;
		if (!checkLists("6b - InRect(0,0,10,10)", pl.findInRectangle(new Rectangle2D.Double(0, 0, 10, 10)), cmp6b)) {
			failed = true;
		}
		Point2D.Double[] cmp6c = {};
		if (!checkLists("6c - InRect(1,4,1,1)", pl.findInRectangle(new Rectangle2D.Double(1, 4, 1, 1)), cmp6c)) {
			failed = true;
		}
		// Fuer weitere Tests


		p = pl.findNearest(tp[5]);
		if ((p == null) || !p.equals(tp[5])) {
			failed = true;
			Out.println("Test 7a - Nearest of " + tp[5] + " is not " + p);
		}
		Point2D.Double q = new Point2D.Double(10, 4);
		p = pl.findNearest(q);
		if ((p == null) || !p.equals(tp[10])) {
			failed = true;
			Out.println("Test 7b - Nearest of " + q + " is not " + p + ", but " + tp[10]);
		}
		q = new Point2D.Double(2, 3);
		p = pl.findNearest(q);
		if ((p == null) || !p.equals(tp[4])) {
			failed = true;
			Out.println("Test 7c - Nearest of " + q + " is not " + p + ", but " + tp[4]);
		}
		// Fuer weitere Tests


		if (!(pl.contains(tp[4]) && pl.remove(tp[4]) && !pl.contains(tp[4]))) {
			failed = true;
			Out.println("Test 8a - Removing " + tp[4] + " failed.");
		}
		if (!(pl.contains(tp[7]) && pl.remove(tp[7]) && !pl.contains(tp[7]))) {
			failed = true;
			Out.println("Test 8b - Removing " + tp[7] + " failed.");
		}
		if (!(pl.contains(tp[9]) && pl.remove(tp[9]) && !pl.contains(tp[9]))) {
			failed = true;
			Out.println("Test 8c - Removing " + tp[9] + " failed.");
		}
		if (!(pl.contains(tp[0]) && pl.remove(tp[0]) && !pl.contains(tp[0]))) {
			failed = true;
			Out.println("Test 8d - Removing " + tp[0] + " failed.");
		}
		// Fuer weitere Tests


		Out.println();
		pl.printAll();
		Out.println();
		if (failed) {
			Out.println("Test failed! *********************************");
		}
		else {
			Out.println("Test OK.");
		}
		Out.println();
	}

	// Fit here for processor speed ****************************
	static final int repeatInsert = 200000;
	static final int repeatContains = 1500;
	static final int repeatRemove = 1500;
	static final int repeatRangeX = 500;
	static final int repeatRangeY = 500;
	static final int repeatRectangle = 5000;
	static final int repeatNearest = 5000;
	// *********************************************************

	static void testPerformance() {
		java.util.Random r = new java.util.Random(12345);
		PointList2D list = createPointList();

		// Test insert
		double[] arrInsX = new double[repeatInsert];
		double[] arrInsY = new double[repeatInsert];
		for (int i = 0; i < repeatInsert; i ++) {
			arrInsX[i] = r.nextDouble() * 1000.0;
			arrInsY[i] = r.nextDouble() * 1000.0;
		}
		long start1 = System.currentTimeMillis();
		for (int i = 0; i < repeatInsert; i ++) {
			new Point2D.Double(arrInsX[i], arrInsY[i]);
		}
		long start2 = System.currentTimeMillis();
		for (int i = 0; i < repeatInsert; i ++) {
			list.insert(new Point2D.Double(arrInsX[i], arrInsY[i]));
		}
		long end = System.currentTimeMillis();
		Out.println("Inserting " + repeatInsert + " points needs " +
						((double)(end - start2 - (start2 - start1))) / repeatInsert +
						" milliseconds (" + (end - start2) + "/" + (start2 - start1) + ")");

		// Test contains
		start1 = System.currentTimeMillis();
		for (int i = 0; i < repeatContains; i ++) {
			new Point2D.Double(arrInsX[(i * 1231)%repeatInsert], arrInsY[(i * 1231)%repeatInsert]);
		}
		start2 = System.currentTimeMillis();
		for (int i = 0; i < repeatContains; i ++) {
			list.contains(new Point2D.Double(arrInsX[(i * 1231)%repeatInsert], arrInsY[(i * 1231)%repeatInsert]));
		}
		end = System.currentTimeMillis();
		Out.println("Contains " + repeatContains + " points needs " +
						((double)(end - start2 - (start2 - start1))) / repeatContains +
						" milliseconds (" + (end - start2) + "/" + (start2 - start1) + ")");

		// Test remove
		start1 = System.currentTimeMillis();
		for (int i = 0; i < repeatRemove; i ++) {
			new Point2D.Double(arrInsX[(i * 1231)%repeatInsert], arrInsY[(i * 1231)%repeatInsert]);
		}
		start2 = System.currentTimeMillis();
		for (int i = 0; i < repeatRemove; i ++) {
			list.remove(new Point2D.Double(arrInsX[(i * 1231)%repeatInsert], arrInsY[(i * 1231)%repeatInsert]));
		}
		end = System.currentTimeMillis();
		Out.println("Remove " + repeatRemove + " points needs " +
						((double)(end - start2 - (start2 - start1))) / repeatRemove +
						" milliseconds (" + (end - start2) + "/" + (start2 - start1) + ")");

		// Test findInRangeX
		start1 = System.currentTimeMillis();
		for (int i = 0; i < repeatRangeX; i ++) {
			double x = arrInsX[(i * 101)%repeatInsert];
			x = arrInsX[(i * 101)%repeatInsert] + arrInsX[i] / 32;
		}
		start2 = System.currentTimeMillis();
		for (int i = 0; i < repeatRangeX; i ++) {
			list.findInRangeX(arrInsX[(i * 101)%repeatInsert],
								arrInsX[(i * 101)%repeatInsert] + arrInsX[i] / 32);
		}
		end = System.currentTimeMillis();
		Out.println("FindInRangeX " + repeatRangeX + " points needs " +
						((double)(end - start2 - (start2 - start1))) / repeatRangeX +
						" milliseconds (" + (end - start2) + "/" + (start2 - start1) + ")");

		// Test findInRangeY
		start1 = System.currentTimeMillis();
		for (int i = 0; i < repeatRangeY; i ++) {
			double y = arrInsY[(i * 101)%repeatInsert];
			y = arrInsY[(i * 101)%repeatInsert] + arrInsY[i] / 32;
		}
		start2 = System.currentTimeMillis();
		for (int i = 0; i < repeatRangeY; i ++) {
			list.findInRangeX(arrInsY[(i * 101)%repeatInsert],
								arrInsY[(i * 101)%repeatInsert] + arrInsY[i] / 32);
		}
		end = System.currentTimeMillis();
		Out.println("FindInRangeY " + repeatRangeY + " points needs " +
						((double)(end - start2 - (start2 - start1))) / repeatRangeY +
						" milliseconds (" + (end - start2) + "/" + (start2 - start1) + ")");

		// Test findInRectangle
		start1 = System.currentTimeMillis();
		for (int i = 0; i < repeatRectangle; i ++) {
			Rectangle2D.Double rect = new Rectangle2D.Double(arrInsX[(i * 101)%repeatInsert],
							arrInsY[(i * 101)%repeatInsert],
							arrInsX[i] / 16,
							arrInsY[i] / 16);
		}
		start2 = System.currentTimeMillis();
		for (int i = 0; i < repeatRectangle; i ++) {
			Rectangle2D.Double rect = new Rectangle2D.Double(arrInsX[(i * 101)%repeatInsert],
							arrInsY[(i * 101)%repeatInsert],
							arrInsX[i] / 16,
							arrInsY[i] / 16);
			list.findInRectangle(rect);
		}
		end = System.currentTimeMillis();
		Out.println("FindInRectangle " + repeatRectangle + " points needs " +
						((double)(end - start2 - (start2 - start1))) / repeatRectangle +
						" milliseconds (" + (end - start2) + "/" + (start2 - start1) + ")");

		// Test findNearest
		start1 = System.currentTimeMillis();
		for (int i = 0; i < repeatNearest; i ++) {
			Point2D.Double point = new Point2D.Double(
							arrInsX[(i * 101)%repeatInsert] + arrInsX[i] / 256,
							arrInsY[(i * 101)%repeatInsert] + arrInsY[i] / 256);
		}
		start2 = System.currentTimeMillis();
		for (int i = 0; i < repeatNearest; i ++) {
			Point2D.Double point = new Point2D.Double(
							arrInsX[(i * 101)%repeatInsert] + arrInsX[i] / 256,
							arrInsY[(i * 101)%repeatInsert] + arrInsY[i] / 256);
			list.findNearest(point);
		}
		end = System.currentTimeMillis();
		Out.println("FindNearest " + repeatNearest + " points needs " +
						((double)(end - start2 - (start2 - start1))) / repeatNearest +
						" milliseconds (" + (end - start2) + "/" + (start2 - start1) + ")");

	}

}